define('extra-formats/extra-formats-plugin', [
	'aloha',
	'aloha/plugin',
	'aloha/state-override',
	'jquery',
	'util/arrays',
	'ui/ui',
	'ui/toggleButton',
	'format/format-plugin',
	'ui/port-helper-multi-split',
	'PubSub',
	'require'
], function (Aloha, plugin, StateOverride, $, Arrays, Ui, ToggleButton, Format, MultiSplitButton, PubSub, require) {
	'use strict';

	var GENTICS = window.GENTICS;

	var commandsByElementPlugin = {
		'testbutton': 'testbutton'
	};

	var componentNameByElementPlugin = {
	};

	var textLevelSemanticsPlugin = {
		'testbutton': {
			'active': true,
			'markup': 'span',
			'class': 'testbutton',
			'name': 'toggleMetaView'
		}
	};

	var blockLevelSemanticsPlugin = {
		'testformat': {
			'active': true,
			'markup': 'h2',
			'class': 'testformat'
		},
		'testformat2': {
			'active': true,
			'markup': 'p',
			'class': 'testformat2'
		}
	};

	var blockLabels = {
		'button.testformat.tooltip': "Test Format Label",
		'button.testformat2.tooltip': "Test Format Label 22",
		'button.testbutton.tooltip': "Testbutton"
	};

	var interchangeableNodeNames = {
	};

	function changeMarkup(button, buttonData) {
		Aloha.Selection.changeMarkupOnSelection(jQuery('<'+buttonData['markup']+'>', {'class': buttonData['class']}));
	}

	function addMarkup(button, buttonData) {
		var formatPlugin = Format;
		var markup = jQuery('<'+buttonData['markup']+'>', {'class': buttonData['class']});
		var rangeObject = Aloha.Selection.rangeObject;

		if ( typeof button === "undefined" || button == "" ) {
			return;
		}

		// check whether the markup is found in the range (at the start of the range)
		var nodeNames = interchangeableNodeNames[markup[0].nodeName] || [markup[0].nodeName];
		var foundMarkup = rangeObject.findMarkup(function() {
			return -1 !== Arrays.indexOf(nodeNames, this.nodeName);
		}, Aloha.activeEditable.obj);

		if (foundMarkup) {
			// remove the markup
			if (rangeObject.isCollapsed()) {
				// when the range is collapsed, we remove exactly the one DOM element
				GENTICS.Utils.Dom.removeFromDOM(foundMarkup, rangeObject, true);
			} else {
				// the range is not collapsed, so we remove the markup from the range
				GENTICS.Utils.Dom.removeMarkup(rangeObject, jQuery(foundMarkup), Aloha.activeEditable.obj);
			}
			updateUiAfterMutation(formatPlugin, rangeObject);
		} else {
			// when the range is collapsed, extend it to a word
			if (rangeObject.isCollapsed()) {
				GENTICS.Utils.Dom.extendToWord(rangeObject);
				if (rangeObject.isCollapsed()) {
					if (StateOverride.enabled()) {
						StateOverride.setWithRangeObject(
							commandsByElementPlugin[button],
							rangeObject,
							function (command, rangeObject) {
								format(formatPlugin, rangeObject, markup);
							}
						);
						return;
					}
				}
			}
			format(formatPlugin, rangeObject, markup);
		}
	}

	/**
	 * Update UI after changed like adding new markup
	 * @param formatPlugin
	 * @param rangeObject
	 */
	function updateUiAfterMutation(formatPlugin, rangeObject) {
		// select the modified range
		rangeObject.select();
		// update Button toggle state. We take 'Aloha.Selection.getRangeObject()'
		// because rangeObject is not up-to-date
		onSelectionChangedPlugin(formatPlugin, Aloha.Selection.getRangeObject());
	}

	function format(formatPlugin, rangeObject, markup) {
		GENTICS.Utils.Dom.addMarkup(rangeObject, markup);
		updateUiAfterMutation(formatPlugin, rangeObject);
	}

	function formatInsideTableWorkaroundPlugin(button, buttonData) {
		var selectedCells = jQuery('.aloha-cell-selected');
		if (selectedCells.length > 0) {
			var cellMarkupCounter = 0;
			selectedCells.each(function () {
				var cellContent = jQuery(this).find('div'),
					cellMarkup = cellContent.find(button);
				if (cellMarkup.length > 0) {
					// unwrap all found markup text
					// <td><b>text</b> foo <b>bar</b></td>
					// and wrap the whole contents of the <td> into <b> tags
					// <td><b>text foo bar</b></td>
					cellMarkup.contents().unwrap();
					cellMarkupCounter++;
				}
				cellContent.contents().wrap('<'+buttonData['markup']+'></'+buttonData['markup']+'>');
			});

			// remove all markup if all cells have markup
			if (cellMarkupCounter === selectedCells.length) {
				selectedCells.find(button).contents().unwrap();
			}
			return true;
		}
		return false;
	}

	/**
	 * This method is executed when we click on block level button
	 * @param formatPlugin
	 * @param button
	 * @param buttonData
	 * @returns {boolean}
	 */
	function blockLevelButtonClickHandlerPlugin(formatPlugin, button, buttonData) {
		if (formatInsideTableWorkaroundPlugin(button, buttonData)) {
			return false;
		}

		changeMarkup(button, buttonData);

		// setting the focus is needed for mozilla to have a working rangeObject.select()
		if (Aloha.activeEditable && jQuery.browser.mozilla) {
			Aloha.activeEditable.obj.focus();
		}

		// triggered for numerated-headers plugin
		if (Aloha.activeEditable) {
			Aloha.trigger( 'aloha-format-block' );
		}
	}

	/**
	 * This method is executed when we click on text level button
	 * @param formatPlugin
	 * @param button
	 * @param buttonData
	 * @returns {boolean}
	 */
	function textLevelButtonClickHandlerPlugin(formatPlugin, button, buttonData) {
		if (formatInsideTableWorkaroundPlugin(button, buttonData)) {
			return false;
		}
		addMarkup(button, buttonData);
		return false;
	}

	/**
	 * Make block level buttons like paragraph or heading 1
	 * @param formatPlugin
	 * @param button
	 * @param buttonData
	 * @returns {{name: *, tooltip: *, iconClass: string, markup: (jQuery|*), click: Function}}
	 */
	function makeBlockLevelButtonPlugin(formatPlugin, button, buttonData) {
		return {
			name: button,
			tooltip: blockLabels['button.'+button+'.tooltip'],
			iconClass: 'aloha-icon ' + 'aloha-large-icon-'+button,
			markup: jQuery('<'+buttonData['markup']+'>', {'class': buttonData['class']}),
			click: function () { return blockLevelButtonClickHandlerPlugin(formatPlugin, button, buttonData); }
		};
	}

	/**
	 * Make text level buttons like underline or bold.
	 * @param formatPlugin
	 * @param button
	 * @param buttonData
	 * @returns {*}
	 */
	function makeTextLevelButtonPlugin(formatPlugin, button, buttonData) {
		var componentName = commandsByElementPlugin[button];
		if (componentNameByElementPlugin.hasOwnProperty(button)) {
			componentName = componentNameByElementPlugin[button];
		}
		// add button to UI
		var component = Ui.adopt('toggleMetaView', ToggleButton, {
			tooltip : blockLabels['button.' + button + '.tooltip'],
			icon: 'aloha-icon aloha-icon-'+button,
			scope: 'Aloha.continuoustext',
			click: function () { return textLevelButtonClickHandlerPlugin(formatPlugin, button, buttonData); }
		});
		return component;
	}

	/**
	 * Adjust toolbar when we select a format or we change the cursor position
	 * @param formatPlugin
	 * @param rangeObject
	 */
	function onSelectionChangedPlugin(formatPlugin, rangeObject) {
		var effectiveMarkup,
			foundMultiSplit, i, j, multiSplitItem;
		jQuery.each(formatPlugin.buttons, function (index, button) {
			var statusWasSet = false;
			var nodeNames = interchangeableNodeNames[button.markup[0].nodeName] || [button.markup[0].nodeName];
			var classNames = button.markup[0].className;
			for (i = 0; i < rangeObject.markupEffectiveAtStart.length; i++) {
				effectiveMarkup = rangeObject.markupEffectiveAtStart[i];
				for (j = 0; j < nodeNames.length; j++) {
					var currentMarkup = null;
					// if there is a class associated with warkup, we add it.
					if (classNames[j] != null) {
						currentMarkup = jQuery('<' + nodeNames[j] + '>', {'class': classNames});
					} else {
						currentMarkup = jQuery('<' + nodeNames[j] + '>');
					}
					if (Aloha.Selection.standardTextLevelSemanticsComparator(effectiveMarkup, currentMarkup)) {
						button.handle.setState(true);
						statusWasSet = true;
					}
				}
			}
			if (!statusWasSet) {
				button.handle.setState(false);
			}
		});

		if (formatPlugin.multiSplitItems.length > 0) {
			foundMultiSplit = false;

			// iterate over the markup elements
			for (i = 0; i < rangeObject.markupEffectiveAtStart.length && !foundMultiSplit; i++) {
				effectiveMarkup = rangeObject.markupEffectiveAtStart[i];

				for (j = 0; j < formatPlugin.multiSplitItems.length && !foundMultiSplit; j++) {
					multiSplitItem = formatPlugin.multiSplitItems[j];

					if (!multiSplitItem.markup) {
						continue;
					}

					// now check whether one of the multiSplitItems fits to the effective markup
					if (Aloha.Selection.standardTextLevelSemanticsComparator(effectiveMarkup, multiSplitItem.markup)) {
						formatPlugin.multiSplitButton.setActiveItem(multiSplitItem.name);
						foundMultiSplit = true;
					}
				}
			}

			if (!foundMultiSplit) {
				formatPlugin.multiSplitButton.setActiveItem(null);
			}
		}
	}

	/**
	 * Check is the format is allowed by config
	 * @param tagname
	 * @param plugin
	 * @param editable
	 * @returns {boolean}
	 */
	function isFormatAllowed(tagname, plugin, editable) {
		var config = plugin.getEditableConfig(editable.obj);
		return jQuery.inArray(tagname, config) > -1;
	}

	/**
	 * Create & register the plugin.
	 */
	return plugin.create('extra-formats', {

		configPlugin: ['testformat', 'testformat2'],

		availableButtonsPlugin: ['testbutton', 'testformat', 'testformat2'],

		hotKey: {
			formatTestButton: 'alt+ctrl+t',
			formatTestFormat: 'alt+ctrl+f'
		},

		/**
		 * Executed on plugin initialization.
		 */
		init: function () {
			var me = this;

			this.initButtons();

			// Allow the attribute 'class' for all markup needed
			this.addClassAttributeToAlohaSetting();

			// Load the css used for this plugin
			this.loadPluginCSS();

			// apply specific configuration if an editable has been activated
			Aloha.bind('aloha-editable-activated',function (e, params) {
				Format.applyButtonConfig(params.editable.obj);

				var createAdder = function (tagname) {
					return function () {
						if (isFormatAllowed(tagname, Format, params.editable)) {
							Format.addMarkup(tagname);
						}
						return false;
					};
				};

				var createChanger = function (tagname) {
					return function () {
						if (isFormatAllowed(tagname, Format, params.editable)) {
							Format.changeMarkup(tagname);
						}
						return false;
					};
				};

				params.editable.obj.bind('keydown.aloha.format',  me.hotKey.formatTestButton,createAdder('testbutton'));
				params.editable.obj.bind('keydown.aloha.format', me.hotKey.formatTestFormat, createChanger('testformat'));
			});
		},

		/**
		 * allowed the attribute 'class' used markups.
		 */
		addClassAttributeToAlohaSetting: function () {
			Aloha.settings.contentHandler.allows.attributes.h1.push('class');
			Aloha.settings.contentHandler.allows.attributes.h2.push('class');
			Aloha.settings.contentHandler.allows.attributes.h3.push('class');
			Aloha.settings.contentHandler.allows.attributes.h4.push('class');
			Aloha.settings.contentHandler.allows.attributes.h5.push('class');
			Aloha.settings.contentHandler.allows.attributes.h6.push('class');
			Aloha.settings.contentHandler.allows.attributes.blockquote.push('class');
			Aloha.settings.contentHandler.allows.attributes.span = ['class'];
		},

		/**
		 * load custom css files
		 * This files contain the new styles and buttons style
		 */
		loadPluginCSS: function () {
			var basePluginUrl = require.toUrl('.');
			var cssLink = $("<link rel='stylesheet' type='text/css' href='"+basePluginUrl+"/../css/styles.css'>");
			$("head").append(cssLink);
		},

		/**
		 * initialize the buttons and register them on floating menu
		 */
		initButtons: function () {
			var that = this;

			this.buttons = {};
			this.multiSplitItems = [];

			jQuery.each(this.availableButtonsPlugin, function(j, button) {

				if (typeof j !== 'number' && typeof button !== 'string') {
					button = j;
				}
				if (textLevelSemanticsPlugin[button] != null && textLevelSemanticsPlugin[button]['active']) {

					Format.buttons[button] = {
						handle: makeTextLevelButtonPlugin(Format, button, textLevelSemanticsPlugin[button]),
						markup: jQuery('<'+textLevelSemanticsPlugin[button]['markup']+'>', {'class': textLevelSemanticsPlugin[button]['class']})
					};
				} else if (blockLevelSemanticsPlugin[button] != null && blockLevelSemanticsPlugin[button]['active']) {
					var item = makeBlockLevelButtonPlugin(Format, button, blockLevelSemanticsPlugin[button]);
					// push the new button into original multiSplitButton
					Format.multiSplitButton.pushItem(item);
				} else {
					Aloha.log('warn', that, 'Button "' + button + '" is not defined in plugin');
				}
			});

			// replace the original onSelectionChanged function
			PubSub.sub('aloha.selection.context-change', function(message) {
				onSelectionChangedPlugin(Format, message.range);
			});
		},

		/**
		 * toString method
		 * @return string
		 */
		toString: function () {
			return 'extra-formats';
		}
	});
});
